#version 330 core

layout(location = 0) out vec4 outColor;

in vec3 worldNormal;
in vec3 worldPosition;
in vec2 texCoords;
in vec3 cameraPosition;
in vec4 shadowMapCoords;

uniform vec3 lightDirection;
uniform int renderShadow;
uniform sampler2DShadow shadowMap;

uniform sampler2D sceneTexture;
uniform int motionBlurEnabled;

uniform float deltaTime;

in vec4 vPrevPosition;
in vec4 vCurrPosition;

uniform vec4 color;

void main(){

	vec3 camera = vec3(0, 0, 0);
	camera = cameraPosition;
	vec3 diffuse = vec3(1, 1, 1);
	vec3 ambient = vec3(0.1, 0.1, 0.1);

	// reflection
	vec3 specular = vec3(1, 1, 1);
	float shininess = 40;

	vec3 diffuseColor = diffuse * color.xyz;
	vec3 ambientColor = ambient * color.xyz;

	vec3 N = normalize(worldNormal);
	vec3 V = normalize(camera - worldPosition);

	
	vec3 L = vec3(1, 1, 1); // direction
	vec3 I = vec3(0.7, 0.7, 0.7); // intensity

	float LN = dot(L, N);
	if(LN < 0)
		LN = 0;



	vec3 diff = ((I * LN) * diffuseColor);

	vec3 spec = specular * LN;

	vec3 H = normalize(V + L);
	float NH = dot(H, N);

	if(NH < 0)
		NH = 0;

	vec3 Ispec =  (I * pow(NH, shininess)) * spec;

	//outColor =  vec4(   max(diff + Ispec, ambientColor)   , 1) ;
	
	/* motion blur */

	if(motionBlurEnabled == 1){

		int MAX_SAMPLES = 50;
	
		vec2 a = (vCurrPosition.xy / vCurrPosition.w) * 0.5 + 0.5;
		vec2 b = (vPrevPosition.xy / vPrevPosition.w) * 0.5 + 0.5;
		vec2 velocity = a - b;

		//uVelocityScale = currentFps / targetFps
		
		velocity *= 3 * ((1/deltaTime) / 60);
	
		vec2 texelSize = 1.0 / vec2(textureSize(sceneTexture, 0));
		float speed = length(velocity / texelSize);
		int nSamples = clamp(int(speed), 1, MAX_SAMPLES);


		vec4 pixelColor = texture(sceneTexture, a );

		//blur
		for (int i = 1; i < nSamples; ++i) {
			vec2 offset = velocity * (float(i) / float(nSamples - 1) - 0.5);
			pixelColor += texture(sceneTexture, a + offset);

		}

		outColor = pixelColor / float(nSamples);
		return;
	}


	/* shadow maps */

	float shadowFactor = 1.0;
	if(renderShadow == 1){

		vec3 sm_uvs = shadowMapCoords.xyz / shadowMapCoords.w;
		float depth_p = shadowMapCoords.z / shadowMapCoords.w;
		float depth_c = texture( shadowMap, sm_uvs);

		if(sm_uvs.y > 1 || sm_uvs.x > 1){
			outColor =  vec4(max(diff + Ispec, ambientColor), color.w);
			return;
		}
			

		if(depth_c < depth_p){
			shadowFactor = 0.1;

			outColor =  vec4(ambientColor, color.w);
			//outColor =  vec4(0,0,0, 0);
			return;
		}		
	}
	outColor =  vec4(max(diff + Ispec, ambientColor), color.w) * vec4(shadowFactor, shadowFactor,shadowFactor,1);	

}