#version 330

layout(location = 0) in vec3  position;
layout(location = 1) in vec3  direction;
layout(location = 2) in float lifetime;

out vec3  newPosition;
out vec3  newDirection;
out float newLifetime;

float random(float);
float random(vec2);
float random(vec3);
float random(vec4);

uniform float deltaTime;
uniform float velocity;

void main(){

	vec3 origin = vec3(0,0,0);

	vec3 rand_start = position * 
		vec3((direction.x), (direction.y), (direction.z));

	
	vec3 pos = position;

	vec3 target = vec3(0,40,0);
	vec3 dir = normalize(target - pos);

	newDirection = direction;
	
	float factor = velocity;
	if(velocity > 20 || velocity < 1){
		factor = 6;
	}

	if(lifetime > 0){
		newPosition = (dir * vec3(deltaTime*factor,deltaTime*factor,deltaTime*factor)) + pos;
		newLifetime = lifetime - (deltaTime * 3);
	}
	else if((lifetime < 0) && (lifetime > -1)){
		newLifetime = -1;
		newPosition = pos;
	}
	else if(lifetime == -1)
	{
		vec3 rand = vec3(random(rand_start.x), 
			random(rand_start.y), random(rand_start.z)); 

		if(rand.y < 1)
		{
		
			vec3 nPos = vec3(rand.x * 2 - 1, 0, rand.z * 2 - 1);
			if(abs(length(nPos)) > 1)
			{
				newPosition  = ((normalize(nPos) * vec3(4.2, 0, 4.2))) + origin; 
				newLifetime  = 5 * random(rand.x * rand.z);	
			}
			else{
				newPosition  = (((nPos) * vec3(3.3, 0, 3.3))) + origin; 
				newLifetime  = 5 * random(rand.x * rand.z);	
			}
		}
	}
	else if(lifetime < -1){
		newPosition = pos;
		newLifetime = lifetime + (deltaTime * 3);
	}
	else{
		newPosition  = pos;
		newLifetime  = lifetime;

	}

}




/* ::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::: */
/* :::::::::::::: RANDOM :::::::::::::::::::::::::::::::::::::::::::::::::::: */
/* ::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::: */


// A single iteration of Bob Jenkins' One-At-A-Time hashing algorithm.
uint hash( uint x ) {
    x += ( x << 10u );
    x ^= ( x >>  6u );
    x += ( x <<  3u );
    x ^= ( x >> 11u );
    x += ( x << 15u );
    return x;
}


// Compound versions of the hashing algorithm I whipped together.
uint hash( uvec2 v ) { return hash( v.x ^ hash(v.y)                         ); }
uint hash( uvec3 v ) { return hash( v.x ^ hash(v.y) ^ hash(v.z)             ); }
uint hash( uvec4 v ) { return hash( v.x ^ hash(v.y) ^ hash(v.z) ^ hash(v.w) ); }


// Construct a float with half-open range [0:1] using low 23 bits.
// All zeroes yields 0.0, all ones yields the next smallest representable value below 1.0.
float floatConstruct( uint m ) {
    const uint ieeeMantissa = 0x007FFFFFu; // binary32 mantissa bitmask
    const uint ieeeOne      = 0x3F800000u; // 1.0 in IEEE binary32

    m &= ieeeMantissa;                     // Keep only mantissa bits (fractional part)
    m |= ieeeOne;                          // Add fractional part to 1.0

    float  f = uintBitsToFloat( m );       // Range [1:2]
    return f - 1.0;                        // Range [0:1]
}


// Pseudo-random value in half-open range [0:1].
float random( vec2  v ) { return floatConstruct(hash(floatBitsToUint(v))); }
float random( vec3  v ) { return floatConstruct(hash(floatBitsToUint(v))); }
float random( vec4  v ) { return floatConstruct(hash(floatBitsToUint(v))); }
float random( float x ) { return floatConstruct(hash(floatBitsToUint(x))); }

