#version 330 core

layout(location = 0) out vec4 outColor;

in vec3 worldNormal;
in vec3 worldPosition;
in vec2 texCoords;
in vec3 cameraPosition;
in vec4 shadowMapCoords;

uniform int renderShadow;
uniform sampler2D color_texture;
uniform sampler2DShadow shadowMap;

void main(){

	vec3 camera = vec3(0, 0, 0);
	//camera = cameraPosition;
	vec3 diffuse = vec3(0.99, 0.99, 0.99);
	vec3 ambient = vec3(0.2, 0.2, 0.2);

	// reflection
	vec3 specular = vec3(1, 1, 1) * vec3(0.5, 0.5, 0.5);
	float shininess = 40;

	vec3 diffuseColor = diffuse * texture(color_texture, texCoords).xyz;
	vec3 ambientColor = ambient * texture(color_texture, texCoords).xyz;

	vec3 N = normalize(worldNormal);
	vec3 V = normalize(camera - worldPosition);

	
	vec3 L = normalize(vec3(1, 1, 1)); // direction
	vec3 I = vec3(0.7, 0.7, 0.7); // intensity

	float LN = normalize(dot(L, N));
	if(LN < 0)
		LN = 0;



	vec3 diff = ((I * LN) * diffuseColor);

	vec3 spec = specular * LN;

	vec3 H = normalize(V + L);
	float NH = dot(H, N);

	if(NH < 0)
		NH = 0;

	vec3 Ispec =  (I * pow(NH, shininess)) * spec;

	//outColor = texture( shadowMap, texCoords);

	/* shadow maps */

	float shadowFactor = 1.0;
	if(renderShadow == 1){
		vec3 sm_uvs = shadowMapCoords.xyz / shadowMapCoords.w;

		//sm_uvs.x =  1.0 - (1.0 - sm_uvs.x) *25.0;
		//sm_uvs.y =  1.0 - (1.0 - sm_uvs.y) *25.0;

		float depth_p = shadowMapCoords.z / shadowMapCoords.w;

		//depth_p =  1.0 - (1.0 - depth_p) *25.0;

		float depth_c = texture( shadowMap, sm_uvs);	
		if(depth_c < depth_p){
			shadowFactor = 0.1;
outColor =  vec4(ambientColor, 1);
return;
		}
			
	}

	/*if(renderShadow){
	vec2 sm_uvs = shadowMapCoords.xyz / shadowMapCoords.w;
	float depth_p = shadowMapCoords.z / shadowMapCoords.w;
	float depth_c = texture( shadowMap, sm_uvs).r;
	if(depth_c < depth_p){
		//shadow
		outColor =  vec4(max(diff + Ispec, ambientColor), 1) ;
	}


	}*/

outColor =  vec4(max(diff + Ispec, ambientColor), 1) * vec4(shadowFactor, shadowFactor,shadowFactor,1);



		

}
